/**
 *  ---------
 * |.##> <##.|  Open Smart Card Development Platform (www.openscdp.org)
 * |#       #|
 * |#       #|  Copyright (c) 1999-2018 CardContact Software & System Consulting
 * |'##> <##'|  Andreas Schwier, 32429 Minden, Germany (www.cardcontact.de)
 *  ---------
 *
 *  This file is part of OpenSCDP.
 *
 *  OpenSCDP is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 *
 *  OpenSCDP is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with OpenSCDP; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @fileoverview Service request controller for ManageRoleServiceRequest
 */

ServiceRequestModel = require('pki-as-a-service/service/ServiceRequestModel').ServiceRequestModel;
Person = require('pki-as-a-service/subjects/Person').Person;
SubjectDAO = require('scsh/pki-db/db/SubjectDAO').SubjectDAO;
CommonUI = require('scsh/srv-cc1/CommonUI').CommonUI;
I18N = require('scsh/srv-cc1/I18N').I18N;




I18N.addResources([
	{ lang: "EN", keys: {
		"msg.mr.title": "Manage Role",
		"msg.mr.assign": "Assign Role to Subject",
		"msg.mr.remove": "Remove Role from Subject",
		"msg.mr.select": "Select Roles",
		"msg.mr.assigned": "Assigned Roles",
		"msg.mr.removed": "Removed Roles",
		"msg.mr.subject": "Subject",
		"msg.mr.id": "ID",
		"msg.mr.roles": "Roles",
		"msg.mr.noAssignableRoles": "No assignable role",
		"msg.mr.noRemovableRoles": "No removable role",
	}}
//	,
//	{ lang: "DE", keys: {
//		"action.save": "Speichern"
//	}}
]);



function ManageRoleServiceRequestController() {
	GPSystem.log(GPSystem.DEBUG, module.id, "new ManageRoleServiceRequestController()");
}

exports.ManageRoleServiceRequestController = ManageRoleServiceRequestController;



ManageRoleServiceRequestController.prototype.getSubject = function(req, res, srct) {
	var parameter = CommonUI.parseQueryString(req.queryString);
	var id = Number(parameter.subject);

	GPSystem.log(GPSystem.DEBUG, module.id, "typeof argument id " + typeof(id));
	GPSystem.log(GPSystem.DEBUG, module.id, "id = " + id);

	if (isNaN(id) || id == 0) {
		throw new GPError(module.id, GPError.INVALID_DATA, 0, "Argument id not valid in URL");
	}

	var dao = srct.service.daof.getSubjectDAO();
	var bo = dao.getSubject(id);
	if (!bo) {
		throw new GPError(module.id, GPError.INVALID_DATA, 0, "No subject found with id " + id);
	}

	var factory = srct.service.pluginRegistry.getFactoryForSubject(bo.type);
	if (!factory) {
		throw new GPError(module.id, GPError.INVALID_DATA, 0, "Can not find factory for subject " + bo.type);
	}

	var subject = factory.getSubject(bo);
	return subject;
}



ManageRoleServiceRequestController.prototype.createRequest = function(srct, subject) {

	var srDAO = srct.service.daof.getServiceRequestDAO();

	if (subject.bo.name) {
		var details = subject.bo.name;
	} else {
		var details = subject.bo.email;
	}

	var t = {
		process: "ManageRoleServiceRequest",
		title: srct.i18n("msg.mr.title"),
		details: details,
		originatorId: srct.ui.session.user.id,
		recipientId: subject.getId(),
		state: "Created",
		content: "{}"
	};

	var bo = srDAO.newServiceRequest(t);
	return bo;
}



ManageRoleServiceRequestController.prototype.postCreateAction = function(req, res, srct, action) {
	var subject = this.getSubject(req, res, srct);
	if (!subject.canRead(srct.ui.session.user)) {
	  return false;
	} else if (!(subject instanceof Person)) {
		return false;
	}

	var bo = this.createRequest(srct, subject);

	srct.ui.redirectTo(req, res, req.url, "/sr/" + bo.id);
	return true;
}



ManageRoleServiceRequestController.prototype.restfulPostProcessAction = function(req, res, srct, sr, action) {
	GPSystem.log(GPSystem.DEBUG, module.id, "restfulPostProcessAction: " + action);
	var s = req.getEntityAsString();
	if (s) {
		try	{
			var reqo = JSON.parse(s);
		}
		catch(e) {
			GPSystem.log(GPSystem.ERROR, module.id, "Error parsing JSON request: " + e);
			res.setStatus(HttpResponse.SC_BAD_REQUEST);
			return;
		}
	} else {
		var reqo = [];
	}
	GPSystem.log(GPSystem.DEBUG, module.id, "getEntityAsString: " +s);

	switch(action) {
	case "action.save":
	case "action.update":
		if (!sr.setFields(srct.ui.session.user, reqo)) {
			GPSystem.log(GPSystem.DEBUG, module.id, "Error setting fields for reqo: " + reqo);
			// TODO return json with invalid fields
			return true;
		}
		break;
	}

	sr.perform(srct.ui.session.user, action);

	return false;
}



ManageRoleServiceRequestController.prototype.postProcessAction = function(req, res, srct, sr, action) {
	print("handleAction: " + action);

	switch(action) {
	case "action.save":
	case "action.update":
		if (!sr.setFields(srct.ui.session.user, req.parameters)) {
			var page = srct.getProcessView(req, res, sr);
			srct.ui.sendPage(req, res, req.url, page);
			return true;
		}
		break;
	}

	sr.perform(srct.ui.session.user, action);

	return false;
}



ManageRoleServiceRequestController.prototype.getCreateView = function(req, res, srct) {
	default xml namespace = "http://www.w3.org/1999/xhtml";

	var parameter = CommonUI.parseQueryString(req.queryString);
	var id = parameter.subject;
	var subjectLink = srct.ui.homeURL(req.url) + "/subject/" + id;

	var form =	<div>
				<h1>Manage Roles</h1>
				<p>This service requests manages the roles of <a href={subjectLink}>{"user " + id}</a>. You can assign new roles and/or remove previously assigned roles if you are authorized to manage the specific role.</p>

				<form class="pure-form" action="" method="post" enctype="multipart/form-data">
					<button class="pure-button" type="submit" name="action" value="create">Start Request</button>
				</form>
			</div>

	return form;
}



ManageRoleServiceRequestController.prototype.getProcessView = function(req, res, srct, sr) {
	default xml namespace = "http://www.w3.org/1999/xhtml";

	var div = <div />;

	div.appendChild(srct.renderForm(req, res, sr));

	return div;
}
